//
//  MasterViewController.swift
//  NoteApp-iPhone-I
//
//  Created by Henry Liu on 9/4/14.
//  Copyright (c) 2014 Henry Liu. All rights reserved.
//

import UIKit
import CoreData

@objc (NoteItem1)
class NoteItem1: NSManagedObject {
    @NSManaged var itemName: String
    @NSManaged var content: String
}

class MasterViewController: UITableViewController, NSFetchedResultsControllerDelegate {

    var managedObjectContext: NSManagedObjectContext? = nil
    var itemNames = NSMutableArray ()
    var itemContents = NSMutableArray ()
    var currentRowIndex = -1
    @IBAction func unwindToMasterView (segue: UIStoryboardSegue) {
        println ("MVC: unwindToMasterView +")
        var source: DetailViewController = segue.sourceViewController as DetailViewController
        if let item = source.noteItem as NoteItem! {
            if self.currentRowIndex < 0 {
                itemNames.addObject(item.itemName)
                itemContents.addObject(item.content)
            } else {
                let currentItem = NoteItem ()
                currentItem.itemName = itemNames [self.currentRowIndex] as String
                currentItem.content = itemContents [self.currentRowIndex] as String
                deleteItem(currentItem)
                
                itemNames.replaceObjectAtIndex(self.currentRowIndex, withObject: item.itemName)
                itemContents.replaceObjectAtIndex(self.currentRowIndex, withObject: item.content)
            }
            saveItem (item)
            self.tableView.reloadData()
        } else {
            println ("MVC: unwindToMasterView item nil")
        }
        println ("MVC: unwindToMasterView -")
    }
    func saveItem (item: NoteItem) {
        println ("MVC: saveItem +")
        let  appDelegate = UIApplication.sharedApplication().delegate as AppDelegate
        let context: NSManagedObjectContext = appDelegate.managedObjectContext!
        let noteEntity = NSEntityDescription.entityForName("Notes", inManagedObjectContext: context)
        
        // insert newNote into context
        var newNote = NoteItem1 (entity: noteEntity!, insertIntoManagedObjectContext: context)
        
        //set note name (don't do newNote.itemName = textView.text - crashes)
        newNote.setValue(item.itemName, forKey: "itemName")
        newNote.setValue(item.content, forKey: "content")
        println ("MVC: saveItem insertObject \(newNote.itemName)")
        context.insertObject(newNote)
        var error: NSError?
        context.save(&error)
        println ("MVC: saveItem -")
    }
    func deleteItem (item: NoteItem) {
        println ("MVC: deleteItem +")
        let  appDelegate = UIApplication.sharedApplication().delegate as AppDelegate
        let context: NSManagedObjectContext = appDelegate.managedObjectContext!
        let noteEntity = NSEntityDescription.entityForName("Notes", inManagedObjectContext: context)
        var request: NSFetchRequest  = NSFetchRequest (entityName: "Notes")
        request.predicate = NSPredicate (format: "itemName == \"\(item.itemName)\"")
        
        var error: NSError?
        var objects = context.executeFetchRequest (request, error:&error)
        
        if (objects?.isEmpty != nil) {
            println ("No data in the database")
        } else {
            for foundItem: NSManagedObject in objects as [NSManagedObject] {
                var itemName = foundItem.valueForKey("itemName") as String
                println ("found: \(itemName) of \(objects?.count)")
            }
            println ("MVC: delete numberOfRowsInSection \(self.tableView.numberOfRowsInSection)")
            println ("MVC: deleteObject")
            context.deleteObject(objects?[0] as NSManagedObject)
            println ("MVC: context save")
            context.save(&error)
        }
    println ("MVC: deleteItem -")
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
    }

    override func viewDidLoad() {
        println ("MVC: viewDidLoad +")
        self.loadInitialData()
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        self.navigationItem.leftBarButtonItem = self.editButtonItem()
        /* overwrite
        let addButton = UIBarButtonItem(barButtonSystemItem: .Add, target: self, action: "insertNewObject:")
        
        self.navigationItem.rightBarButtonItem = addButton
        */
        println ("MVC: viewDidLoad -")
    }
    func loadInitialData () {
        println ("MVC: loadInitialData +")
        let  appDelegate = UIApplication.sharedApplication().delegate as AppDelegate
        let context: NSManagedObjectContext = appDelegate.managedObjectContext!
        let noteEntity = NSEntityDescription.entityForName("Notes", inManagedObjectContext: context)
        
        var request: NSFetchRequest  = NSFetchRequest (entityName: "Notes")
        request.predicate = NSPredicate (format: "itemName != nil")
        
        var error: NSError?
        var objects = context.executeFetchRequest (request, error:&error)
        
        if objects?.isEmpty != nil {
            println ("No data in the database")
        } else {
            for foundItem: NSManagedObject in objects as [NSManagedObject] {
                var itemName = foundItem.valueForKey("itemName") as String
                var content = foundItem.valueForKey("content") as String
                itemNames.addObject (itemName)
                itemContents.addObject(content)
                println ("MVC: \(itemName) for \(content)")
            }
        }
        println ("MVC: loadInitialData -")
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
/* not needed anymore
    func insertNewObject(sender: AnyObject) {
        let context = self.fetchedResultsController.managedObjectContext
        let entity = self.fetchedResultsController.fetchRequest.entity
        let newManagedObject = NSEntityDescription.insertNewObjectForEntityForName(entity.name, inManagedObjectContext: context) as NSManagedObject
             
        // If appropriate, configure the new managed object.
        // Normally you should use accessor methods, but using KVC here avoids the need to add a custom class to the template.
        newManagedObject.setValue("Apple", forKey: "itemName")
        newManagedObject.setValue("content", forKey: "content")
             
        // Save the context.
        var error: NSError? = nil
        if !context.save(&error) {
            // Replace this implementation with code to handle the error appropriately.
            // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            //println("Unresolved error \(error), \(error.userInfo)")
            abort()
        }
    }
*/
    // MARK: - Segues

    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        println ("MVC: prepareForSegue + \(segue.identifier) index = \(self.currentRowIndex)")
        if let indexPath = self.tableView.indexPathForSelectedRow() {
            self.currentRowIndex = indexPath.row
        }
        println ("MVC: prepareForSegue + \(segue.identifier) index = \(self.currentRowIndex)")
        let noteItem = NoteItem ()
        if segue.identifier == "showDetail" {
            let indexPath = self.tableView.indexPathForSelectedRow()
            // let object = self.fetchedResultsController.objectAtIndexPath(indexPath) as NSManagedObject
            // (segue.destinationViewController as DetailViewController).detailItem = object
            self.currentRowIndex = indexPath!.row
            noteItem.itemName = itemNames [self.currentRowIndex] as String
            noteItem.content = itemContents [self.currentRowIndex] as String
            (segue.destinationViewController as DetailViewController).detailItem = noteItem
        } else if segue.identifier == "addItem" {
            self.currentRowIndex = -1
            noteItem.itemName = ""
            noteItem.content = ""
            (segue.destinationViewController as DetailViewController).detailItem = noteItem
        }
        println ("MVC: prepareForSegue - \(segue.identifier) index = \(self.currentRowIndex)")
    }

    // MARK: - Table View

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        //return self.fetchedResultsController.sections.count
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        /*
        let sectionInfo = self.fetchedResultsController.sections[section] as NSFetchedResultsSectionInfo
        println("MVC: number numberOfRowsInsection + \(sectionInfo.numberOfObjects)")
        return sectionInfo.numberOfObjects
        */
        return itemNames.count
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        //println ("MVC: cellForRowAtIndex + \(indexPath.row)")
        let cell = tableView.dequeueReusableCellWithIdentifier("Cell", forIndexPath: indexPath) as UITableViewCell
//        self.configureCell(cell, atIndexPath: indexPath)
        cell.textLabel?.text = itemNames [indexPath.row] as NSString
        //println ("MVC: cellForRowAtIndex - \(indexPath.row)")
        return cell
    }

    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }

    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {
            let itemToDelete = NoteItem ()
            //let selectedItemName: AnyObject! = itemNames [indexPath.row]
            itemToDelete.itemName = itemNames [indexPath.row] as String
            itemToDelete.content = itemContents [indexPath.row] as String
            deleteItem(itemToDelete)
            itemNames.removeObjectAtIndex(indexPath.row)
            itemContents.removeObjectAtIndex(indexPath.row)

            // update table view
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)

            /* default
            let context = self.fetchedResultsController.managedObjectContext
            context.deleteObject(self.fetchedResultsController.objectAtIndexPath(indexPath) as NSManagedObject)
                
            var error: NSError? = nil
            if !context.save(&error) {
                // Replace this implementation with code to handle the error appropriately.
                // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
                //println("Unresolved error \(error), \(error.userInfo)")
                abort()
            }
            */
        }
    }
/*
    func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let object = self.fetchedResultsController.objectAtIndexPath(indexPath) as NSManagedObject
        cell.textLabel.text = object.valueForKey("itemName").description
    }
*/
    // MARK: - Fetched results controller

    var fetchedResultsController: NSFetchedResultsController {
        println ("MVC: fetchedResultsController + ")
        if _fetchedResultsController != nil {
            return _fetchedResultsController!
        }
        
        let fetchRequest = NSFetchRequest()
        // Edit the entity name as appropriate.
        let entity = NSEntityDescription.entityForName("Notes", inManagedObjectContext: self.managedObjectContext!)
        fetchRequest.entity = entity
        
        // Set the batch size to a suitable number.
        fetchRequest.fetchBatchSize = 20
        
        // Edit the sort key as appropriate.
            
        let sortDescriptor = NSSortDescriptor(key: "itemName", ascending: false)
        let sortDescriptors = [sortDescriptor]
        
        fetchRequest.sortDescriptors = [sortDescriptor]
        
        // Edit the section name key path and cache name if appropriate.
        // nil for section name key path means "no sections".
        let aFetchedResultsController = NSFetchedResultsController(fetchRequest: fetchRequest, managedObjectContext: self.managedObjectContext!, sectionNameKeyPath: nil, cacheName: "Master")
        aFetchedResultsController.delegate = self
        _fetchedResultsController = aFetchedResultsController
        
    	var error: NSError? = nil
    	if !_fetchedResultsController!.performFetch(&error) {
    	     // Replace this implementation with code to handle the error appropriately.
    	     // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
             //println("Unresolved error \(error), \(error.userInfo)")
    	     abort()
    	}
        println ("MVC: fetchedResultsController - ")
        return _fetchedResultsController!
    }    
    var _fetchedResultsController: NSFetchedResultsController? = nil

    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        self.tableView.beginUpdates()
    }

    func controller(controller: NSFetchedResultsController, didChangeSection sectionInfo: NSFetchedResultsSectionInfo, atIndex sectionIndex: Int, forChangeType type: NSFetchedResultsChangeType) {
        switch type {
            case .Insert:
                self.tableView.insertSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
            case .Delete:
                self.tableView.deleteSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
            default:
                return
        }
    }

    func controller(controller: NSFetchedResultsController, didChangeObject anObject: AnyObject, atIndexPath indexPath: NSIndexPath, forChangeType type: NSFetchedResultsChangeType, newIndexPath: NSIndexPath) {
        switch type {
            case .Insert:
                tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            case .Delete:
                tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            //case .Update:
             //   self.configureCell(tableView.cellForRowAtIndexPath(indexPath), atIndexPath: indexPath)
            case .Move:
                tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            default:
                return
        }
    }

    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        self.tableView.endUpdates()
    }

    /*
     // Implementing the above methods to update the table view in response to individual changes may have performance implications if a large number of changes are made simultaneously. If this proves to be an issue, you can instead just implement controllerDidChangeContent: which notifies the delegate that all section and object changes have been processed.
     
     func controllerDidChangeContent(controller: NSFetchedResultsController) {
         // In the simplest, most efficient, case, reload the table view.
         self.tableView.reloadData()
     }
     */

}

